/*
 * Copyright (c)1998 by Angry Red Planet.
 *
 * This code is distributed under a modified form of the
 * Artistic License.  A copy of this license should have
 * been included with it; if this wasn't the case, the
 * entire package can be obtained at
 * <URL:http://www.angryredplanet.com/>.
 *
 * ----------------------------------------------------------------------
 *
 * RandomSaver.h
 *
 * A screen saver module that randomly picks another module to show.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 * There is not yet a configuration interface for this module, so you
 * have to live with it selecting from all of your available screen
 * savers.
 *
 * ----------------------------------------------------------------------
 *
 * To Do
 * ~~~~~
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 *
 * July 31, 1999:
 *	First public release.
 *
 */

#include <add-ons/screen_saver/ScreenSaver.h>
#include <storage/FindDirectory.h>
#include <support/String.h>

class RandomSaver : public BScreenSaver
{
public:
						RandomSaver(BMessage *message, image_id id);
	virtual				~RandomSaver();
	
	// Initialize module: get current settings for other modules, and list
	// of all available modules.
	virtual status_t	InitCheck();
	
	// Start and stop the saver.  If 'preview' is not true [i.e., we are
	// really running], then randomly pick another screen saver and actually
	// start up that one.
	virtual status_t	StartSaver(BView *view, bool preview);
	virtual void		StopSaver();
	
	// If not in preview mode, then redirect all drawing calls to the
	// actual screen saver...  otherwise, just don't do anything.
	virtual void		Draw(BView *view, int32 frame);
	virtual void		DirectConnected(direct_buffer_info *info);
	virtual void		DirectDraw(int32 frame);
	
	// Start and stop configuration.  There unfortunately isn't anything to
	// configure at this point...
	virtual void		StartConfig(BView *configView);
	virtual void		StopConfig();
	
	// Supply and receive info.  If not running in preview mode, these are
	// redirected to the actual screen saver...  otherwise, do nothing.
	virtual void		SupplyInfo(BMessage *info) const;
	virtual void		ModulesChanged(const BMessage *info);
	
	// Return configuration of this saver.  Just copy in our mSaverSettings
	// message.
	virtual	status_t	SaveState(BMessage *into) const;
	
private:
	// Add all files in the given directory to the field "name" in 'to'.
	// Note that this checks to make sure that entries in the directory
	// are files and can be opened with load_add_on, but doesn't check
	// any farther than that for actual screen saver modules.
	status_t			AddModules(directory_which path, const char* file,
								   BMessage* to) const;
	
	// Select a random module from mSaverSettings, and load it up.
	status_t			LoadRandomSaver();
	
	// Unload any loaded module.
	void				UnloadSaver();
	
	// Copy the current tick and loop values up from the actual screen saver
	// to our state.
	void				CopyChildState();
	
	// This is information about our own screen saver module.
	image_id			mMyImage;
	BString				mMyName;
	
	// This contains the current configuration.  There are two fields:
	// "name" is an array of all the found screen savers.  This is used
	// for synchronization, to determine if any modules have been added or
	// removed since the last time we were run.
	// "sel" is an array of the modules that are currently selected for
	// random picking.  Since there isn't yet a GUI to change this, it
	// for now should just be all of the available modules.
	BMessage			mMySettings;
	
	// This is a copy of the Be screen saver settings, so we can pass the
	// correct settings into a module we run.  It's an ugly hack.
	BMessage			mSaverSettings;
	
	// Information about the currently running module.
	BString				mRealName;
	BScreenSaver*		mRealSaver;
	image_id			mRealImage;
	bool				mIsPreview;
};
